/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DistributedJMXServer.java 16:40:02 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.communication.jmx;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Set;

import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.QueryExp;
import javax.management.ReflectionException;
import javax.management.remote.JMXConnector;

import org.objectweb.petals.kernel.server.MBeanHelper;

/**
 * This object is a connection to a JMX server
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class DistributedJMXServer {
    
    /**
     * The Petals JMX Domain
     */
    public static final String PETALS_DOMAIN = "Petals";

    /**
     * Name of the method getServiceDescription
     */
    public static final String GET_SERVICE_DESCRITION = "getServiceDescription";

    /**
     * Name of the method isExchangeWithConsumerOkayForComponent
     */
    public static final String IS_OK_WITH_CONS = "isExchangeWithConsumerOkayForComponent";

    /**
     * Name of the method isExchangeWithProviderOkayForComponent
     */
    public static final String IS_OK_WITH_PROV = "isExchangeWithProviderOkayForComponent";

    /**
     * Connection to the JMX server
     */
    private MBeanServerConnection connection;

    /**
     * JMX Connector
     */
    private JMXConnector connector;

    /**
     * Creates a new instance of {@link JMXConnector}
     * 
     * @param connector
     */
    public DistributedJMXServer(JMXConnector connector) {
        this.connector = connector;
    }
    
    /**
     * Creates a new instance of {@link JMXConnector}
     * 
     * @param connector
     */
    public DistributedJMXServer(MBeanServerConnection connection) {
        this.connection = connection;
    }

    /**
     * 
     * @return
     */
    protected MBeanServerConnection getMBeanServerConnection() {
        if (connection == null) {
            try {
                connector.connect();
                connection = connector.getMBeanServerConnection();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return this.connection;
    }

    /**
     * Retrieve the object name of the admin service
     * 
     * @return object name
     * @throws IOException
     * @throws NullPointerException
     * @throws MalformedObjectNameException
     */
    public ObjectName getAdminServiceMBeanName() throws IOException,
        MalformedObjectNameException, NullPointerException {
        if (connection == null) {
            connector.connect();
            connection = connector.getMBeanServerConnection();
        }
        ObjectName result = null;
        Set objNames;
        Hashtable<String, String> attrs = new Hashtable<String, String>();
        attrs.put("name", MBeanHelper.ADMIN_MBEAN);
        attrs.put("type", "service");
        ObjectName objName = new ObjectName(PETALS_DOMAIN, attrs);
        objNames = connection.queryNames(objName, null);
        if (objNames != null && objNames.size() == 1) {
            result = (ObjectName) objNames.iterator().next();
        }
        return result;
    }

    /**
     * 
     * @param name
     * @param operationName
     * @param params
     * @param signature
     * @return
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws ReflectionException
     * @throws IOException
     */
    public Object invoke(ObjectName name, String operationName,
            Object[] params, String[] signature)
        throws InstanceNotFoundException, MBeanException, ReflectionException,
        IOException {
        if (connection == null) {
            connector.connect();
            connection = connector.getMBeanServerConnection();
        }
        Object out = null;
        out = connection.invoke(name, operationName, params, signature);
        return out;
    }

    /**
     * 
     * @param name
     * @param queryExp
     * @return
     * @throws InstanceNotFoundException
     * @throws MBeanException
     * @throws ReflectionException
     * @throws IOException
     */
    public Set queryNames(ObjectName name, QueryExp queryExp)
        throws InstanceNotFoundException, MBeanException, ReflectionException,
        IOException {
        if (connection == null) {
            connector.connect();
            connection = connector.getMBeanServerConnection();
        }
        Set out = null;
        out = connection.queryNames(name, queryExp);
        return out;
    }

    /**
     * 
     * @throws IOException
     */
    public void closeConnector() throws IOException {
        if (connector != null) {
            connector.close();
        }
    }
}
