/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JNDIResponse.java 10:55:58 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.communication.jndi.agent.msg.response;

import java.io.Serializable;

/**
 * Registry response
 *
 * @author ddesjardins - eBMWebsourcing
 * @author Roland NAUDIN - eBMWebsourcing
 */
public class RegistryResponse implements Serializable {

    /**
     * Serial UID
     */
    private static final long serialVersionUID = 6179754274097458153L;

    /**
     * Types of response
     *
     */
    public static enum ResponseType {
        /*
         * ACK :
         */
        ack,
        /*
         * CREATE SUBCONTEXT : arg1 = name of the newly created context
         */
        createSubcontext,
        /*
         * EXCEPTION : arg1 = naming exception
         */
        exception,
        /*
         * LIST : arg1 = array of NameClassPair
         */
        list,
        /*
         * LIST BINDINGS : arg1 = array of Binding
         */
        listBindings,
        /*
         * LOOKUP : arg1 = value of the lookup object
         */
        lookup,
        /*
         * LOOKUP LINK: arg1 = value of the lookup object
         */
        lookupLink
    }

    /**
     * First argument of the response
     */
    protected Object arg1;

    /**
     * Response type
     */
    protected ResponseType type;

    /**
     * Creates a new instance of {@link RegistryResponse}
     *
     * @param type
     * @param sender
     * @param recipient
     */
    protected RegistryResponse(ResponseType type) {
        super();
        this.type = type;
    }

    /**
     * Creates a new instance of {@link RegistryResponse}
     *
     * @param type
     * @param arg1
     * @param sender
     * @param recipient
     */
    protected RegistryResponse(ResponseType type, Object arg1) {
        super();
        this.type = type;
        this.arg1 = arg1;
    }

    public Object getArg1() {
        return arg1;
    }

    public ResponseType getType() {
        return type;
    }

    public void setArg1(Object arg1) {
        this.arg1 = arg1;
    }

    public void setType(ResponseType type) {
        this.type = type;
    }

    @Override
    public String toString() {
        return "RegistryResponse " + type;
    }

}
