/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.communication.network;

import java.util.HashMap;
import java.util.Map;

/**
 * The interface of the Network service. It provides the communication amongst
 * distributed PEtALS servers of the same domain.<br>
 * The service proposes to notify a coordinator about the joining or leaving of
 * a PEtALS node. <br>
 * It provides a replicated HashMap object which can be used by the PEtALS JNDI
 * server.
 *
 * @author Roland NAUDIN - eBMWebSourcing
 * @since Petals 2.0
 */
public interface NetworkService {

    static final String JOIN_ACCEPTED = "accepted";

    /**
     * Ask to join the network. The coordinator of the network will decide to
     * accept or not the new node.
     *
     * @return  {@code NetworkService.JOIN_ACCEPTED} if the node is allow to join the network,
     *         otherwise the cause of the refuse
     * @throws NetworkException
     *             Network failure
     */
    String joinNetwork() throws NetworkException;

    /**
     * Leave the network. <br>
     * NOTE: if the response of the joinNetwork() request is {@false}, this method must
     * be not called.
     *
     */
    void leaveNetwork();

    /**
     * Get the HashMap replicated amongst the PEtALS network.
     *
     * @return the replicated hashMap
     */
    Map<String, HashMap<String, Object>> getReplicatedHashMap();
}
