/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.communication.topology;

import java.util.Set;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.kernel.configuration.ContainerConfiguration;

/**
 * The interface of the Topology service. The service is connected to the JNDI
 * service and handles the topology information. At its start, it creates all
 * the local resource needed to be match the topology; it registers the local
 * container configuration on the JNDI server, it builds the JOrAM configuration
 * file<br>
 * In dynamic mode, when a new container join the domain, it verifies the
 * validity of its configuration against topology information. It adds the new
 * JOrAM server to the JOrAM configuration.<br>
 * It provides a method to get all the containers configuration belonging to the
 * domain.
 *
 * @author Roland NAUDIN - eBMWebSourcing
 * @since Petals 2.0
 */
public interface TopologyService {

    /**
     * Get all the containers configuration with the given state.<br>
     * If the state is {@code null}, returns all the containers information.
     *
     * @param state
     *            get only the containers with the given state. if {@code null},
     *            all containers are retrieved
     * @return a set of {@link ContainerConfiguration}
     * @throws PetalsException
     *             Failed to retrieve the containers configuration
     */
    Set<ContainerConfiguration> getAllContainersConfiguration(
        ContainerConfiguration.ContainerState state) throws PetalsException;

    /**
     * Get the container configuration of the given container name.
     *
     * @param containerName
     *            The name of the targeted container
     * @return The container configuration
     * @throws PetalsException
     *             Failed to retrieve the container configuration
     */
    ContainerConfiguration getContainerConfiguration(String containerName)
        throws PetalsException;

    /**
     * Set the given state to the give container
     *
     * @param containerName
     * @param state
     *            The {@code ontainerConfiguration.ContainerState}
     * @throws PetalsException
     *             Failed to set the state to the container
     */
    void setContainerState(String containerName,
        ContainerConfiguration.ContainerState state) throws PetalsException;

    /**
     * Test if the container is started.
     *
     * @param containerName
     *            The container name
     * @return {@code true} if the container is started
     * @throws PetalsException
     *             Failed to retrieve the container configuration
     */
    boolean isContainerStarted(String containerName) throws PetalsException;

    /**
     * Unregister the given container.
     *
     * @throws PetalsException
     */
    void unregisterContainer(String containerName) throws PetalsException;
}
