/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: ComponentLifeCycle.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.petals.container.lifecycle;

import java.io.File;
import java.net.URI;
import java.util.Map;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.management.ObjectName;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.Service;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.container.thread.ComponentLifeCycleThread;
import org.objectweb.petals.jbi.component.context.ComponentContextImpl;
import org.objectweb.petals.jbi.component.context.ComponentInitialContext;
import org.objectweb.petals.jbi.descriptor.ComponentDescription;
import org.objectweb.petals.jbi.descriptor.JBIDescriptor;
import org.objectweb.petals.jbi.management.admin.AdminServiceMBean;
import org.objectweb.petals.jbi.management.endpoint.EndpointService;
import org.objectweb.petals.jbi.management.task.TaskContextConstants;
import org.objectweb.petals.jbi.management.task.installation.InstallationContextConstants;
import org.objectweb.petals.jbi.management.task.installation.InstallationUtils;
import org.objectweb.petals.jbi.messaging.endpoint.ConsumerEndpoint;
import org.objectweb.petals.jbi.messaging.exchange.DeliveryChannelImpl;
import org.objectweb.petals.jbi.messaging.registry.Registry;
import org.objectweb.petals.jbi.messaging.routing.Router;
import org.objectweb.petals.kernel.configuration.ConfigurationService;
import org.objectweb.petals.kernel.configuration.ContainerConfiguration;
import org.objectweb.petals.platform.repository.RepositoryService;
import org.objectweb.petals.platform.systemstate.SystemStateService;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Implementation of the <code>ComponentLifeCycle</code> mbean
 *
 * @author Adrien LOUIS - EBM WebSourcing
 * @author wjoseph - eBM-WebSourcing
 * @author nsalatge - eBM-WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.objectweb.petals.container.lifecycle.ComponentLifeCycleMBean.class))
public class ComponentLifeCycle extends LifeCycleAbstract implements
    ComponentLifeCycleMBean, ComponentContextCommunication {

    /**
     * The component linked with the current component life cycle
     */
    protected Component component;

    /**
     * The component description
     */
    private ComponentDescription componentDescription;

    /**
     * The component context
     */
    private ComponentContextImpl componentContext;

    /**
     * The component life cycle thread. Each life cycle have a thread to process
     * operation in the component ClassLoader
     */
    protected ComponentLifeCycleThread componentLifeCycleThread;

    /**
     * Platform component :: SystemState Service
     */
    @Requires(name = "systemstate", signature = org.objectweb.petals.platform.systemstate.SystemStateService.class)
    private SystemStateService systemStateService;

    /**
     * Platform Component :: Repository Service
     */
    @Requires(name = "repository", signature = org.objectweb.petals.platform.repository.RepositoryService.class)
    protected RepositoryService repositoryService;

    /**
     * The endpointService is used to manage all the endpoint interfaces in a
     * distributed context
     */
    @Requires(name = "endpoint", signature = org.objectweb.petals.jbi.management.endpoint.EndpointService.class)
    protected EndpointService endpointService;

    /**
     * JBI-Messaging component :: JNDIRegistry Service
     */
    @Requires(name = "registry", signature = org.objectweb.petals.jbi.messaging.registry.Registry.class)
    protected Registry registryService;

    protected Map<String, Object> context;

    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * JBI-Management Component :: Admin Service
     */
    @Requires(name = "admin", signature = org.objectweb.petals.jbi.management.admin.AdminServiceMBean.class)
    protected AdminServiceMBean adminService;

    /**
     * JBI-Messaging Component :: Router Service
     */
    @Requires(name = "router", signature = org.objectweb.petals.jbi.messaging.routing.Router.class)
    protected Router router;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration", signature = org.objectweb.petals.kernel.configuration.ConfigurationService.class)
    private ConfigurationService configurationService;

    /**
     * The container configuration
     */
    private ContainerConfiguration containerConfiguration;


    protected JBIDescriptor descriptor;

    protected String installationRootPath;

    protected String workingRootPath;

    protected ConsumerEndpoint address;

    protected InitialContext initialContext;

    /**
     * The component
     */
    @Service
    private org.objectweb.fractal.api.Component comp;

    /**
     * Creates a new instance of {@link ComponentLifeCycle}
     *
     */
    public ComponentLifeCycle() {
        super();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.jbi.component.lifecycle.ComponentLifeCycleMBean#init(org.objectweb.petals.tools.jbicommon.descriptor.ComponentDescription,
     *      javax.jbi.component.Component, javax.management.ObjectName,
     *      org.objectweb.petals.jbi.component.thread.ComponentLifeCycleThread,
     *      org.objectweb.petals.util.LoggingUtil, java.util.Map)
     */
    public void init(ComponentDescription componentDescription,
        Component component, ObjectName mbeanName,
        ComponentLifeCycleThread componentLifeCycleThread, LoggingUtil log,
        Map<String, Object> context) throws PetalsException {

        log.start();

        super.init(mbeanName, log);

        this.componentDescription = componentDescription;

        this.component = component;

        this.context = context;

        this.descriptor = (JBIDescriptor) context
            .get(TaskContextConstants.DESCRIPTOR);

        URI installRoot = (URI) context.get(TaskContextConstants.INSTALL_ROOT);

        String componentName = InstallationUtils
            .getComponentNameFromDescriptor(this.descriptor);

        this.installationRootPath = new File(installRoot).getAbsolutePath();

        this.workingRootPath = repositoryService
            .getComponentWorkRoot(componentName).getAbsolutePath();

        this.address = new ConsumerEndpoint(componentName, containerConfiguration
            .getName());

        this.initialContext = createComponentInitialContext();

        // 1. create the component context
        this.componentContext = createComponentContext();

        // set the jbiComponent ref to the contextImpl
        if (this.componentContext != null) {
            this.componentContext.setComponent(component);
        }

        // life cycle thread
        this.componentLifeCycleThread = componentLifeCycleThread;

        log.end();
    }

    /**
     * Create and return the component context
     *
     * @return the component context
     * @throws PetalsException
     */
    private ComponentContextImpl createComponentContext()
        throws PetalsException {
        ComponentContextImpl componentContextImpl = new ComponentContextImpl(
            comp);
        context.put(InstallationContextConstants.COMPONENT_CONTEXT,
            componentContextImpl);

        return componentContextImpl;
    }

    /**
     * The initial JNDI context for all component. If this initial context is
     * null, it will be created
     *
     * @return the initial JNDI context for all component. If this initial
     *         context is null, it will be created
     * @throws PetalsException
     *             if {@link InitialContext} can't be created
     */
    private InitialContext createComponentInitialContext()
        throws PetalsException {
        ComponentInitialContext componentInitialContext = null;
        try {
            componentInitialContext = new ComponentInitialContext(
                registryService.getUsersContext());
        } catch (NamingException e) {
            String msg = "Error creating InitialContext for components";
            log.error(msg, e);
            throw new PetalsException(msg, e);
        }
        return componentInitialContext;
    }

    @Override
    public void doInit() throws JBIException {
        log.start();
        componentLifeCycleThread.doInit(componentContext);
        log.end();
    }

    @Override
    public void doShutdown() throws JBIException {
        log.start();
        componentLifeCycleThread.doShutdown();

        // close the Channel if required
        DeliveryChannelImpl channel = componentContext.getDeliveryChannelImpl();
        if (channel != null && channel.isOpened())
            channel.close();

        log.end();
    }

    @Override
    public void doStart() throws JBIException {
        log.start();
        containerConfiguration = configurationService.getContainerConfiguration();
        componentLifeCycleThread.doStart();
        log.end();
    }

    @Override
    public void doStop() throws JBIException {
        log.start();
        componentLifeCycleThread.doStop();
        log.end();
    }

    /**
     * @see javax.jbi.management.ComponentLifeCycleMBean#getExtensionMBeanName()
     */
    public ObjectName getExtensionMBeanName() throws JBIException {
        log.start();
        ObjectName on = componentLifeCycleThread.getExtensionMBeanName();
        log.end();
        return on;
    }

    /**
     *
     * @return
     */
    public String getName() {
        return componentDescription.getIdentification().getName();
    }

    @Override
    public void setState(String state) throws JBIException {
        super.setState(state);
        try {
            systemStateService.updateComponentLifeCycleState(getName(), state);
        } catch (Exception e) {
            String msg = "Component state can't be persisted for recovery";
            log.error(msg, e);
            throw new JBIException(msg, e);
        }
    }

    /* getters */

    public Component getComponent() {
        return component;
    }

    public ComponentContextImpl getComponentContext() {
        return componentContext;
    }

    public ComponentDescription getComponentDescription() {
        return componentDescription;
    }

    /**
     * Get the consumer endpoint
     *
     * @return the consumer endpoint
     */
    public ConsumerEndpoint getAddress() {
        return this.address;
    }

    /**
     * Get the endpoint service
     *
     * @return the endpoint service
     */
    public EndpointService getEndpointService() {
        return this.endpointService;
    }

    /**
     * Get the initial context
     *
     * @return the initial context
     */
    public InitialContext getInitialContext() {
        return this.initialContext;
    }

    /**
     * Get the installation root
     *
     * @return the installation root
     */
    public String getInstallationRoot() {
        return this.installationRootPath;
    }

    /**
     * Get the JBI descriptior
     *
     * @return the JBI descriptor
     */
    public JBIDescriptor getJBIDescriptor() {
        return this.descriptor;
    }

    /**
     * Get the admin service
     *
     * @return the admin service
     */
    public AdminServiceMBean getAdminService() {
        return this.adminService;
    }

    /**
     * Get the router
     *
     * @return the router
     */
    public Router getRouter() {
        return this.router;
    }

    /**
     * Get the workspace root
     *
     * @return the workspace root
     */
    public String getWorkspaceRoot() {
        return this.workingRootPath;
    }

    /**
     * Get the logger
     *
     * @return the logger
     */
    public Logger getLogger() {
        return this.logger;
    }

    /**
     * Initialization of the petals component
     *
     * @throws PetalsException
     * @throws JBIException
     *
     */
    @LifeCycle(on = LifeCycleType.START)
    public void startComponentLifeCycle() {
        log = new LoggingUtil(logger, logger.getName().substring(
            logger.getName().lastIndexOf(".") + 1));
        log.start();

        containerConfiguration = configurationService.getContainerConfiguration();

        log.end();
    }

    /**
     * Stop of the petals component
     *
     */
    @LifeCycle(on = LifeCycleType.STOP)
    public void stopComponentLifeCycle() {
        log.start();
        log.end();
        logger = null;
    }
}
