/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.task;

import java.io.File;
import java.net.URI;

import org.objectweb.petals.jbi.management.autoload.AutoLoaderService;
import org.objectweb.petals.processor.Task;

/**
 * Abstract Tack class for classes whose manipulate archive files.
 * 
 * @author ofabre
 * 
 */
public abstract class AbstractFileManipulationTask implements Task {

    private final AutoLoaderService autoLoaderService;

    public AbstractFileManipulationTask(AutoLoaderService autoLoaderService) {
        this.autoLoaderService = autoLoaderService;
    }

    /**
     * Get the file according to the archive URI
     * 
     * @param archiveURI
     *            URI of the archive
     * @return a File pointing on the archive
     */
    public static File getArchiveFile(URI archiveURI) {
        return new File(archiveURI);
    }

    /**
     * Get a file of the given name in the "installed directory"
     * 
     * @param file
     *            the file to look for
     * @return a File pointing on the file name in the "installed directory".
     * 
     */
    public File getInstalledArchive(File file) {
        return new File(autoLoaderService.getInstalledDirectory(), file
            .getName());
    }

    /**
     * Get a file of the given name in the "uninstalled directory"
     * 
     * @param file
     *            the file to look for
     * @return a File pointing on the file name in the "Uninstalled directory".
     *         <code>null</code> if the "installed directory is not defined.
     */
    public File getUninstalledArchive(File file) {
        if (autoLoaderService.getUninstalledDirectory() == null) {
            return null;
        }
        return new File(autoLoaderService.getUninstalledDirectory(), file
            .getName());
    }

    /**
     * Retrieve work directory from AutoloaderServiceImpl
     * 
     * @return the work directory file
     */
    public File getWorkDirectory() {
        return autoLoaderService.getWorkDirectory();
    }

}
