/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.task;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;

import org.apache.commons.io.FileUtils;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.autoload.AutoLoaderService;
import org.objectweb.petals.util.LoggingUtil;

/**
 * This task copy entity (component, shared lib, service assembly) package into
 * the work directory<br/> Inputs : archive {@link URI} <br/> Outputs :
 * {@link File} pointing on unzipped archive <br/>
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class CopyPackageToWorkDirTask extends AbstractFileManipulationTask {

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    /**
     * DateFormat used for repository folders
     */
    private SimpleDateFormat folderDateFormat;

    public CopyPackageToWorkDirTask(AutoLoaderService autoLoaderSrv, LoggingUtil log) {
        super(autoLoaderSrv);
        this.log = log;
        String datePattern = "MM-dd-yyyy-hhmmss";
        folderDateFormat = new SimpleDateFormat(datePattern);

    }

    @SuppressWarnings("unchecked")
    public void execute(HashMap context) throws Exception {

        URI archiveURI = (URI) context.get(TaskContextConstants.ARCHIVE_URI);

        /*
         * Copy package to work dir
         */
        File unzipRoot = copyEntityPackage(archiveURI);

        /*
         * Fill context
         */
        context.put(TaskContextConstants.UNZIP_ROOT, unzipRoot);

    }

    private File copyEntityPackage(URI archiveURI) throws PetalsException {
        File archiveFile = new File(archiveURI);

        File destFile = new File(getWorkDirectory(), archiveFile.getName()
            + "-"
            + folderDateFormat.format(new Date(System.currentTimeMillis())));

        try {
            FileUtils.copyDirectory(archiveFile, destFile);
        } catch (IOException e) {
            throw new PetalsException(
                "Can't copy entity package to work directory", e);
        }
        return destFile;
    }

    public void undo(HashMap context) {
        File tempDir = (File) context.get(TaskContextConstants.UNZIP_ROOT);

        try {
            FileUtils.forceDelete(tempDir);
        } catch (IOException e) {
            String msg = "Failed to revert a ExtractTask";
            log.error(msg, e);
        }
    }

}
