/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.task;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.autoload.AutoLoaderService;
import org.objectweb.petals.util.LoggingUtil;

/**
 * This task download entity (component, shared lib, service assembly) archive
 * into the work directory, only if the URL protocol is http: or ftp: <br/>
 * Inputs : archive {@link URI} <br/> Outputs : downloaded archive {@link URI}
 * <br/>
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class DownloadTask extends AbstractFileManipulationTask {

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    /**
     * DateFormat used for repository folders
     */
    private SimpleDateFormat folderDateFormat;

    public DownloadTask(AutoLoaderService autoLoaderSrv, LoggingUtil log) {
        super(autoLoaderSrv);
        this.log = log;
        String datePattern = "MM-dd-yyyy-hhmmss";
        folderDateFormat = new SimpleDateFormat(datePattern);

    }

    @SuppressWarnings("unchecked")
    public void execute(HashMap context) throws Exception {

        URI archiveURI = (URI) context.get(TaskContextConstants.ARCHIVE_URI);

        /*
         * Explode Comp
         */
        File unzipRoot = downloadEntityPackage(archiveURI);

        /*
         * Fill context
         */
        context.put(TaskContextConstants.ARCHIVE_URI, unzipRoot.toURI());

    }

    private File downloadEntityPackage(URI archiveURI) throws PetalsException {
        InputStream inputStream = null;
        File outputFile = null;
        try {
            inputStream = archiveURI.toURL().openStream();

            outputFile = new File(getWorkDirectory(), "downloaded-"
                + folderDateFormat.format(new Date(System.currentTimeMillis()))
                + ".zip");
            FileOutputStream fileOutputStream = new FileOutputStream(outputFile);
            IOUtils.copy(inputStream, fileOutputStream);
        } catch (MalformedURLException e) {
            // Nothing to do URL already checked;
        } catch (IOException e) {
            new PetalsException("Can't read distant file: " + archiveURI);
        }
        return outputFile;
    }

    public void undo(HashMap context) {
        URI downloadURI = (URI) context.get(TaskContextConstants.ARCHIVE_URI);

        try {
            FileUtils.forceDelete(new File(downloadURI));
        } catch (IOException e) {
            String msg = "Failed to revert a DownloadTask";
            log.error(msg, e);
        }
    }

}
