/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ExtractSUsTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.task.deployment.deploy;

import java.io.File;
import java.net.URI;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.objectweb.petals.jbi.descriptor.JBIDescriptor;
import org.objectweb.petals.jbi.descriptor.ServiceUnit;
import org.objectweb.petals.jbi.management.ManagementException;
import org.objectweb.petals.jbi.management.task.TaskContextConstants;
import org.objectweb.petals.jbi.management.task.deployment.DeploymentContextConstants;
import org.objectweb.petals.jbi.management.util.PackageHandler;
import org.objectweb.petals.platform.repository.RepositoryService;
import org.objectweb.petals.processor.Task;
import org.objectweb.petals.util.LoggingUtil;

/**
 * This task renames service unit packages into the repository to fits with
 * service unit ids in the jbi descriptor of the service assembly <br/> Inputs :
 * {@link JBIDescriptor}, parent service assembly install root {@link URI}
 * <br/> Outputs : SU install roots {@link Map} <br/>
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class RenameSUsTask implements Task {

    /**
     * Package Handler
     */
    protected PackageHandler packageHandler;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    public RenameSUsTask(PackageHandler packageHandler,
        RepositoryService repositoryService, LoggingUtil log) {
        super();
        this.packageHandler = packageHandler;
        this.log = log;
    }

    @SuppressWarnings("unchecked")
    public void execute(HashMap context) throws Exception {
        JBIDescriptor descriptor = (JBIDescriptor) context
            .get(TaskContextConstants.DESCRIPTOR);

        URI saInstallRoot = (URI) context
            .get(TaskContextConstants.INSTALL_ROOT);

        /*
         * Explodes SUs
         */
        Map<String, URI> suInstallRoots = renameSUS(saInstallRoot, descriptor);

        /*
         * Fill context
         */
        context
            .put(DeploymentContextConstants.SU_INSTALL_ROOTS, suInstallRoots);

    }

    /**
     * Explode service unit packages into the repository
     * 
     * @param saInstallRoot
     *            the root {@link URI} of the service assembly
     * @param descriptor
     *            the service assembly descriptor
     * @return the map containing all su install roots
     * @throws ManagementException
     *             if the repository service doesn't manage to expend su archive
     */
    protected Map<String, URI> renameSUS(URI saInstallRoot,
        JBIDescriptor descriptor) throws ManagementException {
        Map<String, URI> suInstallRoots = new HashMap<String, URI>();
        List<ServiceUnit> sus = descriptor.getServiceAssembly()
            .getServiceUnits();
        for (ServiceUnit unit : sus) {
            File suPackageFile = getSUPackageFile(unit, saInstallRoot);
            File destFile = new File(saInstallRoot.getPath(), unit
                .getIdentification().getName());
            suPackageFile.renameTo(destFile);
            suInstallRoots.put(unit.getIdentification().getName(), destFile
                .toURI());
        }
        return suInstallRoots;
    }

    /**
     * Retrieve the service unit package {@link File}
     * 
     * @param su
     *            the {@link ServiceUnit} production element
     * @param saInstallRoot
     *            the sa installation root {@link URI}
     * @return the service unit package {@link File}
     */
    protected File getSUPackageFile(ServiceUnit su, URI saInstallRoot) {
        return new File(saInstallRoot.getPath(), su.getTargetArtifactsZip());
    }

    public void undo(HashMap context) {
        // Nothing to do
    }

}
