/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: InstallationTaskFactory.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.task.installation;

import org.objectweb.petals.container.ContainerService;
import org.objectweb.petals.jbi.management.admin.AdminServiceMBean;
import org.objectweb.petals.jbi.management.autoload.AutoLoaderService;
import org.objectweb.petals.jbi.management.task.CopyArchiveToInstalledDirTask;
import org.objectweb.petals.jbi.management.task.CopyPackageToWorkDirTask;
import org.objectweb.petals.jbi.management.task.DownloadTask;
import org.objectweb.petals.jbi.management.task.ExtractTask;
import org.objectweb.petals.jbi.management.task.SuccessTask;
import org.objectweb.petals.jbi.management.task.installation.install.component.CheckInstalledSLForCompTask;
import org.objectweb.petals.jbi.management.task.installation.install.component.ComponentCheckPackageTask;
import org.objectweb.petals.jbi.management.task.installation.install.component.CopyComponentPackageToRepositoryTask;
import org.objectweb.petals.jbi.management.task.installation.install.component.CreateAndRegisterInstallerMBeanTask;
import org.objectweb.petals.jbi.management.task.installation.install.component.CreateComponentStateHolderTask;
import org.objectweb.petals.jbi.management.task.installation.install.sl.CopySLPackageToRepositoryTask;
import org.objectweb.petals.jbi.management.task.installation.install.sl.CreateSLClassloaderTask;
import org.objectweb.petals.jbi.management.task.installation.install.sl.CreateSLStateHolderTask;
import org.objectweb.petals.jbi.management.task.installation.install.sl.SLCheckPackageTask;
import org.objectweb.petals.jbi.management.task.installation.uninstall.component.DeleteArchiveFromInstalledDirTask;
import org.objectweb.petals.jbi.management.task.installation.uninstall.component.RemoveComponentFolderTask;
import org.objectweb.petals.jbi.management.task.installation.uninstall.component.RemoveComponentStateHolderTask;
import org.objectweb.petals.jbi.management.task.installation.uninstall.component.UnregisterInstallerTask;
import org.objectweb.petals.jbi.management.task.installation.uninstall.sl.CheckComponentShutdownStateTask;
import org.objectweb.petals.jbi.management.task.installation.uninstall.sl.RemoveSLClassloaderTask;
import org.objectweb.petals.jbi.management.task.installation.uninstall.sl.RemoveSLFolderTask;
import org.objectweb.petals.jbi.management.task.installation.uninstall.sl.RemoveSLStateHolderTask;
import org.objectweb.petals.jbi.management.util.PackageHandler;
import org.objectweb.petals.jbi.messaging.routing.Router;
import org.objectweb.petals.platform.classloader.ClassLoaderService;
import org.objectweb.petals.platform.repository.RepositoryService;
import org.objectweb.petals.platform.systemstate.SystemStateService;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * This Factory creates Installation Tasks.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class InstallationTaskFactory {

    private CheckInstalledSLForCompTask checkInstalledSLForCompTask;

    private CopyComponentPackageToRepositoryTask copyComponentPackageToRepositoryTask;

    private ComponentCheckPackageTask componentCheckPackageTask;

    private CreateComponentStateHolderTask createComponentStateHolderTask;

    private CreateAndRegisterInstallerMBeanTask createAndRegisterInstallerMBeanTask;

    private CreateSLClassloaderTask createSLClassloaderTask;

    private CopySLPackageToRepositoryTask copySLPackageToRepositoryTask;

    private SLCheckPackageTask slCheckPackageTask;

    private CreateSLStateHolderTask createSLStateHolderTask;

    private CopyArchiveToInstalledDirTask copyArchiveToInstalledDirTask;

    private RemoveComponentFolderTask removeComponentFolderTask;

    private RemoveComponentStateHolderTask removeComponentStateHolderTask;

    private DeleteArchiveFromInstalledDirTask deleteCompArchiveFromInstalledDirTask;

    private UnregisterInstallerTask unregisterInstallerTask;

    private CheckComponentShutdownStateTask checkComponentShutdownStateTask;

    private org.objectweb.petals.jbi.management.task.installation.uninstall.sl.DeleteArchiveFromInstalledDirTask deleteSLArchiveFromInstalledDirTask;

    private RemoveSLClassloaderTask removeSLClassloaderTask;

    private RemoveSLFolderTask removeSLFolderTask;

    private RemoveSLStateHolderTask removeSLStateHolderTask;

    private SuccessTask successTask;

    private ExtractTask extractTask;

    private CopyPackageToWorkDirTask copyPackageToWorkDirTask;

    private DownloadTask downloadTask;

    /**
     * Creates a new instance of {@link InstallationTaskFactory}
     * 
     * @param packageHandler
     * @param systemStateService
     * @param repositorySrv
     * @param log
     * @param logger
     * @param adminService
     * @param router
     * @param loaderService
     */
    public InstallationTaskFactory(PackageHandler packageHandler,
        SystemStateService systemStateService, RepositoryService repositorySrv,
        LoggingUtil log, Logger logger, AdminServiceMBean adminService,
        Router router, ClassLoaderService loaderService,
        ContainerService container, AutoLoaderService autoLoaderSrv) {
        super();

        /*
         * Create Tasks
         */
        // Component creation tasks
        checkInstalledSLForCompTask = new CheckInstalledSLForCompTask(
            systemStateService, log);

        copyComponentPackageToRepositoryTask = new CopyComponentPackageToRepositoryTask(
            repositorySrv, log);

        componentCheckPackageTask = new ComponentCheckPackageTask(log,
            packageHandler, adminService);

        createComponentStateHolderTask = new CreateComponentStateHolderTask(
            systemStateService, repositorySrv, autoLoaderSrv, log);

        createAndRegisterInstallerMBeanTask = new CreateAndRegisterInstallerMBeanTask(
            log, adminService, router, container);

        successTask = new SuccessTask(autoLoaderSrv, log);

        // SL creation tasks
        createSLClassloaderTask = new CreateSLClassloaderTask(log,
            loaderService, adminService, container);

        copySLPackageToRepositoryTask = new CopySLPackageToRepositoryTask(
            repositorySrv, log);

        slCheckPackageTask = new SLCheckPackageTask(log, packageHandler,
            adminService);

        createSLStateHolderTask = new CreateSLStateHolderTask(
            systemStateService, repositorySrv, autoLoaderSrv, log);

        copyArchiveToInstalledDirTask = new CopyArchiveToInstalledDirTask(
            autoLoaderSrv, log);

        // Component removal tasks
        removeComponentFolderTask = new RemoveComponentFolderTask(repositorySrv);

        removeComponentStateHolderTask = new RemoveComponentStateHolderTask(
            systemStateService);

        deleteCompArchiveFromInstalledDirTask = new DeleteArchiveFromInstalledDirTask(
            autoLoaderSrv, packageHandler);

        unregisterInstallerTask = new UnregisterInstallerTask(adminService,
            container);

        // SL removal tasks
        checkComponentShutdownStateTask = new CheckComponentShutdownStateTask(
            adminService, log);

        deleteSLArchiveFromInstalledDirTask = new org.objectweb.petals.jbi.management.task.installation.uninstall.sl.DeleteArchiveFromInstalledDirTask(
            autoLoaderSrv, packageHandler);

        removeSLClassloaderTask = new RemoveSLClassloaderTask(adminService,
            container);

        removeSLFolderTask = new RemoveSLFolderTask(repositorySrv);

        removeSLStateHolderTask = new RemoveSLStateHolderTask(
            systemStateService);

        extractTask = new ExtractTask(autoLoaderSrv, log);

        copyPackageToWorkDirTask = new CopyPackageToWorkDirTask(autoLoaderSrv,
            log);

        downloadTask = new DownloadTask(autoLoaderSrv, log);
    }

    public DownloadTask getDownloadTask() {
        return downloadTask;
    }

    public CopyPackageToWorkDirTask getCopyArchiveToWorkDirTask() {
        return copyPackageToWorkDirTask;
    }

    public CheckInstalledSLForCompTask getCheckInstalledSLForCompTask() {
        return checkInstalledSLForCompTask;
    }

    public CopyComponentPackageToRepositoryTask getCopyComponentPackageToRepositoryTask() {
        return copyComponentPackageToRepositoryTask;
    }

    public ComponentCheckPackageTask getComponentCheckPackageTask() {
        return componentCheckPackageTask;
    }

    public CreateComponentStateHolderTask getCreateComponentStateHolderTask() {
        return createComponentStateHolderTask;
    }

    public CopyArchiveToInstalledDirTask getCopyArchiveToInstalledDirTask() {
        return copyArchiveToInstalledDirTask;
    }

    public CreateAndRegisterInstallerMBeanTask getCreateAndRegisterInstallerMBeanTask() {
        return createAndRegisterInstallerMBeanTask;
    }

    public CreateSLClassloaderTask getCreateSLClassloaderTask() {
        return createSLClassloaderTask;
    }

    public CopySLPackageToRepositoryTask getCopySLPackageToRepositoryTask() {
        return copySLPackageToRepositoryTask;
    }

    public SLCheckPackageTask getSlCheckPackageTask() {
        return slCheckPackageTask;
    }

    public CreateSLStateHolderTask getCreateSLStateHolderTask() {
        return createSLStateHolderTask;
    }

    public RemoveComponentFolderTask getComponentFolderRemovalTask() {
        return removeComponentFolderTask;
    }

    public CheckComponentShutdownStateTask getComponentShutdownStateCheckingTask() {
        return checkComponentShutdownStateTask;
    }

    public RemoveComponentStateHolderTask getComponentStateHolderRemovalTask() {
        return removeComponentStateHolderTask;
    }

    public DeleteArchiveFromInstalledDirTask getDeleteCompArchiveFromInstalledDirTask() {
        return deleteCompArchiveFromInstalledDirTask;
    }

    public org.objectweb.petals.jbi.management.task.installation.uninstall.sl.DeleteArchiveFromInstalledDirTask getDeleteSLArchiveFromInstalledDirTask() {
        return deleteSLArchiveFromInstalledDirTask;
    }

    public UnregisterInstallerTask getDeregisterInstallerTask() {
        return unregisterInstallerTask;
    }

    public RemoveSLClassloaderTask getRemoveSLClassloaderTask() {
        return removeSLClassloaderTask;
    }

    public RemoveSLFolderTask getRemoveSLFolderTask() {
        return removeSLFolderTask;
    }

    public RemoveSLStateHolderTask getRemoveSLStateHolderTask() {
        return removeSLStateHolderTask;
    }

    public CheckComponentShutdownStateTask getCheckComponentShutdownStateTask() {
        return checkComponentShutdownStateTask;
    }

    public CopyComponentPackageToRepositoryTask getExtractComponentTask() {
        return copyComponentPackageToRepositoryTask;
    }

    public RemoveComponentFolderTask getRemoveComponentFolderTask() {
        return removeComponentFolderTask;
    }

    public RemoveComponentStateHolderTask getRemoveComponentStateHolderTask() {
        return removeComponentStateHolderTask;
    }

    public SuccessTask getSuccessTask() {
        return successTask;
    }

    public ExtractTask getExtractTask() {
        return extractTask;
    }

}
