/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.task.installation.install.component;

import java.util.HashMap;
import java.util.List;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.descriptor.JBIDescriptor;
import org.objectweb.petals.jbi.descriptor.SharedLibraryList;
import org.objectweb.petals.jbi.management.task.TaskContextConstants;
import org.objectweb.petals.platform.systemstate.SystemStateService;
import org.objectweb.petals.processor.Task;
import org.objectweb.petals.util.LoggingUtil;

/**
 * This task checks installed shared lib for component : a shared lib must be
 * installed before a component can be installed that makes use of the shared
 * lib <br/>
 * Inputs : {@link JBIDescriptor} <br/>
 * 
 * @author ofabre
 * 
 */
public class CheckInstalledSLForCompTask implements Task {

    /**
     * logger wrapper
     */
    private LoggingUtil log;

    private SystemStateService systemStateService;

    public CheckInstalledSLForCompTask(
        SystemStateService systemStateService, LoggingUtil log) {
        super();
        this.systemStateService = systemStateService;
        this.log = log;
    }

    public void execute(HashMap context) throws Exception {

        JBIDescriptor descriptor = (JBIDescriptor) context
            .get(TaskContextConstants.DESCRIPTOR);

        /*
         * Check installed shared lib for component : a shared lib must be
         * installed before a component can be installed that makes use of the
         * shared lib
         */
        checkInstalledSLForComp(descriptor);

    }

    /**
     * Check installed shared lib for component
     * 
     * @param descriptor
     *            the component <code>JBIDescriptor</code>
     * @throws PetalsException
     *             if a shared library is missing for the component being
     *             installed
     */
    protected void checkInstalledSLForComp(JBIDescriptor descriptor)
        throws PetalsException {
        List<SharedLibraryList> sls = descriptor.getComponent()
            .getSharedLibraryList();

        if (sls != null) {
            for (SharedLibraryList list : sls) {
                if (!systemStateService.isSLLoaded(list.getName())) {
                    String msg = "Component depends on a not installed shared library : "
                        + list.getName();

                    log.error(msg);
                    throw new PetalsException(msg);
                }
            }
        }
    }

    public void undo(HashMap context) throws Exception {
        // Nothing to do
    }

}
