/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.task.installation.uninstall.component;

import java.util.HashMap;

import javax.management.ObjectName;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.container.ContainerService;
import org.objectweb.petals.container.lifecycle.Installer;
import org.objectweb.petals.jbi.management.admin.AdminServiceMBean;
import org.objectweb.petals.jbi.management.task.TaskContextConstants;
import org.objectweb.petals.jbi.management.task.installation.InstallationContextConstants;
import org.objectweb.petals.processor.Task;

/**
 * This task deregister component installer from JMX server<br/> Inputs :
 * component name <br/> Outputs : component installer {@link ObjectName}
 *
 * @author ofabre
 *
 */
public class UnregisterInstallerTask implements Task {

    private AdminServiceMBean jmxAdmin;
    private ContainerService containerService;

    public UnregisterInstallerTask(AdminServiceMBean jmxAdmin, ContainerService container) {
        super();
        this.jmxAdmin = jmxAdmin;
        this.containerService = container;
    }

    @SuppressWarnings("unchecked")
    public void execute(HashMap context) throws Exception {

        String componentName = (String) context
        .get(TaskContextConstants.ENTITY_NAME);
        ObjectName instName = jmxAdmin.getInstallerByName(componentName);
        Installer installer = jmxAdmin.getInstallerMBeanByName(instName);

        if (installer != null && !installer.isInstalled()) {
            jmxAdmin.unregisterInstaller(instName);


            /*
             * Remove of the fractal installer component
             */
            containerService.removeInstaller(componentName);

            try {
                installer.shutdown();
            }
            catch (RuntimeException e) {
                // Do nothing
            }

        } else {
            throw new PetalsException("Can't unload Installer because "
                    + "it's null or its state isn't UNINSTALLED");
        }

        context.put(InstallationContextConstants.INSTALLER_OBJECT_NAME,
                instName);

    }

    public void undo(HashMap context) throws Exception {
        // Nothing to do

    }

}
