/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsAdminService.java 16:40:52 alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.kernel.admin;

import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.naming.Binding;
import javax.naming.InitialContext;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.communication.jndi.client.JNDIService;
import org.objectweb.petals.communication.topology.TopologyService;
import org.objectweb.petals.jbi.management.deployment.DeploymentServiceMBean;
import org.objectweb.petals.jbi.management.installation.InstallationServiceMBean;
import org.objectweb.petals.jbi.messaging.endpoint.AbstractEndpoint;
import org.objectweb.petals.jbi.messaging.registry.JNDIRegistry;
import org.objectweb.petals.jbi.messaging.registry.Registry;
import org.objectweb.petals.jbi.messaging.registry.RegistryException;
import org.objectweb.petals.jbi.messaging.transport.Transporter;
import org.objectweb.petals.jbi.messaging.transport.util.JoramControl;
import org.objectweb.petals.kernel.configuration.ConfigurationService;
import org.objectweb.petals.kernel.configuration.ContainerConfiguration;
import org.objectweb.petals.kernel.configuration.DomainConfiguration;
import org.objectweb.petals.kernel.server.PetalsServer;
import org.objectweb.petals.topology.generated.Configuration;
import org.objectweb.petals.topology.generated.Container;
import org.objectweb.petals.topology.generated.Containers;
import org.objectweb.petals.topology.generated.Domain;
import org.objectweb.petals.topology.generated.DomainMode;
import org.objectweb.petals.topology.generated.DreamService;
import org.objectweb.petals.topology.generated.JmxService;
import org.objectweb.petals.topology.generated.Jndi;
import org.objectweb.petals.topology.generated.JoramService;
import org.objectweb.petals.topology.generated.Multicast;
import org.objectweb.petals.topology.generated.NetworkService;
import org.objectweb.petals.topology.generated.Topology;
import org.objectweb.petals.util.JNDIUtil;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * This class is used for Petals global Administration
 *
 * @author alouis - eBMWebsourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.objectweb.petals.kernel.admin.PetalsAdminServiceInterface.class))
public class PetalsAdminServiceImpl implements PetalsAdminServiceInterface {

    /**
     * Logger wrapper
     */
    protected LoggingUtil log;

    /**
     * Monolog
     */
    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * The Petals Server instance, which manages the Fractal composite
     */
    private PetalsServer petalsServer;

    /**
     * The JOrAM control singleton
     */
    private JoramControl joramControl = JoramControl.getInstance();

    /**
     * Deployment service fractal component
     */
    @Requires(name = "deployment", signature = org.objectweb.petals.jbi.management.deployment.DeploymentServiceMBean.class)
    protected DeploymentServiceMBean deployment;

    /**
     * Installation service fractal component
     */
    @Requires(name = "installation", signature = org.objectweb.petals.jbi.management.installation.InstallationServiceMBean.class)
    protected InstallationServiceMBean installation;

    /**
     * Transporter service fractal component
     */
    @Requires(name = "transporter", signature = org.objectweb.petals.jbi.messaging.transport.Transporter.class)
    protected Transporter transporter;

    /**
     * Topology service fractal component
     */
    @Requires(name = "topology", signature = org.objectweb.petals.communication.topology.TopologyService.class)
    protected TopologyService topologyService;

    /**
     * Registry service fractal component
     */
    @Requires(name = "registry", signature = org.objectweb.petals.jbi.messaging.registry.Registry.class)
    protected Registry registry;

    /**
     * JNDI Service fractal component
     */
    @Requires(name = "jndi", signature = org.objectweb.petals.communication.jndi.client.JNDIService.class)
    protected JNDIService jndiService;
    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration", signature = org.objectweb.petals.kernel.configuration.ConfigurationService.class)
    private ConfigurationService configurationService;

    /**
     * The container configuration
     */
    private ContainerConfiguration containerConfiguration;

    /**
     * The domain configuration
     */
    private DomainConfiguration domainConfiguration;

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.kernel.admin.PetalsAdminServiceMBean#retrieveContainersConfiguration()
     */
    public Set<Map<String, String>> retrieveContainersConfiguration()
        throws PetalsException {
        log.start();

        Set<Map<String, String>> containersSet = new HashSet<Map<String, String>>();

        Map<String, String> domainMap = this.getDomainMap();
        containersSet.add(domainMap);

        Set<ContainerConfiguration> containersConfiguration = retrieveContainersConfiguration(false);

        for (ContainerConfiguration container : containersConfiguration) {
            Map<String, String> containerMap = getContainerMap(container);
            containersSet.add(containerMap);
        }

        log.end();
        return containersSet;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.kernel.admin.PetalsAdminServiceMBean#retrieveStartedContainersConfiguration()
     */
    public Set<Map<String, String>> retrieveStartedContainersConfiguration()
        throws PetalsException {
        log.start();

        Set<Map<String, String>> containersSet = new HashSet<Map<String, String>>();

        Map<String, String> domainMap = this.getDomainMap();
        containersSet.add(domainMap);

        Set<ContainerConfiguration> containersConfiguration = retrieveContainersConfiguration(true);

        for (ContainerConfiguration container : containersConfiguration) {
            Map<String, String> containerMap = getContainerMap(container);
            containersSet.add(containerMap);
        }

        log.end();
        return containersSet;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.kernel.admin.PetalsAdminServiceMBean#removeServerFromNetwork(java.lang.String)
     */
    public void removeServerFromNetwork(String serverName)
        throws PetalsException {
        log.start();

        try {
            Collection<AbstractEndpoint> eps = getServiceEndpointForContainer(serverName);

            for (AbstractEndpoint endpoint : eps) {
                registry.unregisterInternalEndpoint(endpoint);
            }
        } catch (NamingException e) {
            throw new PetalsException(
                "Server not found in the network configuration.", e);
        } catch (RegistryException e) {
            throw new PetalsException(
                "A ServiceEndpoint failed to be removed.", e);
        }

        topologyService.unregisterContainer(serverName);

        joramControl.removeContainer(containerConfiguration);

        log.end();
    }

    public void setPetalsServer(PetalsServer petalsServer) {
        this.petalsServer = petalsServer;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.kernel.admin.PetalsAdminServiceMBean#shutdownContainer()
     */
    public void shutdownContainer() throws Exception {
        log.start();
        log.info("Shutdown the server.");

        if (domainConfiguration.getMode().equals(
            DomainConfiguration.DomainMode.DYNAMIC)) {
            log.info("  1/5 Undeploy Service Assemblies.");
        } else {
            log.info("  1/3 Undeploy Service Assemblies.");
        }

        try {
            deployment.shutdown();
        } catch (Exception e) {
            log.error("Can not shut down the Deployment service.", e);
        }

        if (domainConfiguration.getMode().equals(
            DomainConfiguration.DomainMode.DYNAMIC)) {
            log.info("  2/5 Uninstall Shared Libraries and Components.");
        } else {
            log.info("  2/3 Uninstall Shared Libraries and Components.");
        }

        try {
            installation.shutdown();
        } catch (Exception e) {
            log.error("Can not shut down the Installation service.", e);
        }

        try {
            if (domainConfiguration.getMode().equals(
                DomainConfiguration.DomainMode.DYNAMIC)) {
                log.info("  3/5 Shutdown the Transporter.");
                transporter.shutdown();
            }
        } catch (Exception e) {
            log.error("Can not shut down the Transporter service.", e);
        }

        if (domainConfiguration.getMode().equals(
            DomainConfiguration.DomainMode.DYNAMIC)) {
            log.info("  4/5 Remove the PEtALS node from the domain.");
            topologyService.unregisterContainer(containerConfiguration
                .getName());
        }

        if (domainConfiguration.getMode().equals(
            DomainConfiguration.DomainMode.DYNAMIC)) {
            log.info("  5/5 Stopping the server...");
        } else {
            log.info("  3/3 Stopping the server...");
        }

        // launch the stop of the Petals server in a different thread
        petalsServer.getPetalsStopThread().start();

        log.end();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.kernel.admin.PetalsAdminServiceMBean#stopContainer()
     */
    public void stopContainer() throws Exception {
        log.start();
        log.info("Stopping the server...");
        // launch the stop of the Petals server in a different thread
        petalsServer.getPetalsStopThread().start();
        log.end();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.kernel.admin.PetalsAdminServiceInterface#getDynamicTopology()
     */
    public Topology getDynamicTopology() throws PetalsException {
        log.start();

        if (!domainConfiguration.getMode().equals(
            DomainConfiguration.DomainMode.DYNAMIC)) {
            throw new PetalsException("The domain is not in dynamic mode");
        }

        Multicast multicast = new Multicast();
        multicast.setIp(domainConfiguration.getNetworkMuticastIp());
        multicast.setPort(domainConfiguration.getNetworkMuticastPort());

        Jndi jndi = new Jndi();
        jndi.setFactory(domainConfiguration.getJndiFactory());
        jndi.setHost(domainConfiguration.getJndiHost());
        jndi.setPort(domainConfiguration.getJndiPort());

        Domain domain = new Domain();
        domain.setName(domainConfiguration.getDomain());
        domain.setMode(DomainMode.DYNAMIC);
        domain.setMulticast(multicast);
        domain.setJndi(jndi);

        Containers containers = new Containers();
        Set<ContainerConfiguration> containersConfiguration = retrieveContainersConfiguration(false);

        for (ContainerConfiguration remoteContainerConfiguration : containersConfiguration) {

            NetworkService networkService = new NetworkService();
            networkService.setPort(remoteContainerConfiguration
                .getNetworkBindingPort());

            JmxService jmxService = new JmxService();
            jmxService.setUser(remoteContainerConfiguration.getJmxLogin());
            jmxService.setPassword(remoteContainerConfiguration.getJmxPassword());
            jmxService.setPassword(remoteContainerConfiguration
                .getJmxPassword());
            jmxService.setRmiport(remoteContainerConfiguration
                .getJmxRMIConnectorPort());
            jmxService.setHttpport(remoteContainerConfiguration
                .getJmxHTTPConnectorPort());

            JoramService joramService = new JoramService();
            joramService.setId(remoteContainerConfiguration.getJoramId());
            joramService.setUser(remoteContainerConfiguration.getJoramLogin());
            joramService.setPassword(remoteContainerConfiguration
                .getJoramPassword());

            joramService.setDomainPort(remoteContainerConfiguration
                .getJoramDomainPort());
            joramService.setTcpPort(remoteContainerConfiguration
                .getJoramTCPPort());

            DreamService dreamService = new DreamService();
            dreamService
                .setPort(remoteContainerConfiguration.getDreamTCPPort());

            Configuration configuration = new Configuration();
            configuration.setNetworkservice(networkService);
            configuration.setJmxservice(jmxService);
            configuration.setJoramservice(joramService);
            configuration.setDreamservice(dreamService);

            Container container = new Container();
            container.setName(remoteContainerConfiguration.getName());
            container.setHost(remoteContainerConfiguration.getHost());
            container.setConfiguration(configuration);

            List<Container> containerList = containers.getContainer();
            containerList.add(container);
        }

        Topology topology = new Topology();
        topology.setDomain(domain);
        topology.setContainers(containers);

        log.end();
        return topology;
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws IllegalLifeCycleException {
        log = new LoggingUtil(logger);
        log.start();

        joramControl = JoramControl.getInstance();
        containerConfiguration = configurationService
            .getContainerConfiguration();
        domainConfiguration = configurationService.getDomainConfiguration();

        log.end();
    }

    /**
     * Return all the registered end-points for the specified container
     *
     * @param containerName
     * @return a collection of AbstractEndpoint. Not null, may be empty
     * @throws NamingException
     *             problem accessing JNDI
     */
    protected Collection<AbstractEndpoint> getServiceEndpointForContainer(
        String containerName) throws NamingException {
        log.start();

        InitialContext rootContext = jndiService.getInitialContext();
        Collection<AbstractEndpoint> eps = new HashSet<AbstractEndpoint>();

        if (JNDIUtil.isBound(rootContext, "/", JNDIRegistry.ENDPOINTS_REF)) {

            NamingEnumeration<Binding> enums = rootContext
                .listBindings(JNDIRegistry.ENDPOINTS_REF);

            while (enums.hasMoreElements()) {

                Binding binding = enums.next();
                AbstractEndpoint abstractEndpoint = (AbstractEndpoint) binding
                    .getObject();

                if (!abstractEndpoint.getContainerName().equals(containerName)) {
                    eps.add(abstractEndpoint);
                }
            }
        }

        if (JNDIUtil.isBound(rootContext, "/", JNDIRegistry.NEW_ENDPOINTS_REF)) {

            NamingEnumeration<Binding> enums = rootContext
                .listBindings(JNDIRegistry.NEW_ENDPOINTS_REF);

            while (enums.hasMoreElements()) {

                Binding binding = enums.next();
                AbstractEndpoint abstractEndpoint = (AbstractEndpoint) binding
                    .getObject();

                if (!abstractEndpoint.getContainerName().equals(containerName)) {
                    eps.add(abstractEndpoint);
                }
            }
        }
        log.end();

        return eps;
    }

    /**
     * Retrieve the collection of all the servers configuration of this domain.
     *
     * @param startedOnly
     *            return only started servers information
     * @return Set of server information ContainerInformation
     * @throws PetalsException
     */
    private Set<ContainerConfiguration> retrieveContainersConfiguration(
        boolean startedOnly) throws PetalsException {
        log.start();

        Set<ContainerConfiguration> containers = null;

        if (startedOnly) {
            containers = topologyService
                .getAllContainersConfiguration(ContainerConfiguration.ContainerState.STARTED);
        } else {
            containers = topologyService.getAllContainersConfiguration(null);
        }

        log.end();
        return containers;
    }

    /**
     * Get the domain configuration in key/value form.
     *
     * @return
     */
    private Map<String, String> getDomainMap() {
        Map<String, String> domainMap = new Hashtable<String, String>();
        domainMap.put(CONF_DOMAIN_NAME, domainConfiguration.getDomain());
        if (domainConfiguration.getMode().equals(
            DomainConfiguration.DomainMode.DYNAMIC)) {
            domainMap.put(CONF_DOMAIN_MODE, "dynamic");
        } else if (domainConfiguration.getMode().equals(
            DomainConfiguration.DomainMode.STATIC)) {
            domainMap.put(CONF_DOMAIN_MODE, "static");
        } else {
            domainMap.put(CONF_DOMAIN_MODE, "standalone");
        }
        domainMap.put(CONF_NETWORK_MULTICAST_IP, domainConfiguration
            .getNetworkMuticastIp());
        domainMap.put(CONF_NETWORK_MULTICAST_PORT, Integer
            .toString(domainConfiguration.getNetworkMuticastPort()));
        domainMap.put(CONF_JNDI_FACTORY, domainConfiguration.getJndiFactory());
        if (domainConfiguration.getJndiHost() != null) {
            domainMap.put(CONF_JNDI_HOST, domainConfiguration.getJndiHost());
        }
        if (domainConfiguration.getJndiContainer() != null) {
            domainMap.put(CONF_JNDI_HOST, domainConfiguration
                .getJndiContainer());
        }
        if (domainConfiguration.getJndiPort() != 0) {
            domainMap.put(CONF_JNDI_PORT, Integer.toString(domainConfiguration
                .getJndiPort()));
        }
        return domainMap;
    }

    /**
     * Get the container configuration in key/value form.
     *
     * @param container
     * @return
     */
    private Map<String, String> getContainerMap(ContainerConfiguration container) {
        Map<String, String> containerMap = new Hashtable<String, String>();
        containerMap.put(CONF_CONTAINER_NAME, container.getName());
        containerMap.put(CONF_HOST, container.getHost());
        containerMap.put(CONF_NETWORK_BIND_PORT, Integer.toString(container
            .getNetworkBindingPort()));
        if (container.getJmxLogin() != null) {
            containerMap.put(CONF_JMX_LOGIN, container.getJmxLogin());
        }
        containerMap.put(CONF_JMX_RMI_PORT, Integer.toString(container
            .getJmxRMIConnectorPort()));
        containerMap.put(CONF_JMX_HTTP_PORT, Integer.toString(container
            .getJmxHTTPConnectorPort()));
        containerMap.put(CONF_JORAM_ID, Integer
            .toString(container.getJoramId()));
        containerMap.put(CONF_JORAM_LOGIN, container.getJoramLogin());
        containerMap.put(CONF_JORAM_DOMAIN_PORT, Integer.toString(container
            .getJoramDomainPort()));
        containerMap.put(CONF_JORAM_TCP_PORT, Integer.toString(container
            .getJoramTCPPort()));
        containerMap.put(CONF_DREAM_TCP_PORT, Integer.toString(container
            .getDreamTCPPort()));
        if (container.getState().equals(
            ContainerConfiguration.ContainerState.STARTED)) {
            containerMap.put(CONF_STATE, STATE_STARTED);
        } else {
            containerMap.put(CONF_STATE, STATE_STOPPED);
        }

        return containerMap;
    }
}
