/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PetalsAdminServiceMBean.java 16:40:52 alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.admin;

import java.util.Map;
import java.util.Set;

import org.objectweb.petals.PetalsException;

/**
 * This class is a manager of JMX server. It is used to get a connection to a
 * JMX Server on a distant container.
 *
 * @author ddesjardins - eBMWebsourcing
 */
public interface PetalsAdminServiceMBean {

    /**
     * Domain keys
     */
    public static final String CONF_DOMAIN_NAME = "domainName";

    public static final String CONF_DOMAIN_MODE = "domainMode";

    public static final String CONF_NETWORK_MULTICAST_IP = "multicastIp";

    public static final String CONF_NETWORK_MULTICAST_PORT = "multicastPort";

    public static final String CONF_JNDI_FACTORY = "jndiFactory";

    public static final String CONF_JNDI_HOST = "jndiHost";

    public static final String CONF_JNDI_PORT = "jndiPort";

    /**
     * Container keys
     */
    public static final String CONF_CONTAINER_NAME = "containerName";

    public static final String CONF_HOST = "host";

    public static final String CONF_NETWORK_BIND_PORT = "networkBindPort";

    public static final String CONF_JMX_LOGIN = "jmxLogin";

    public static final String CONF_JMX_RMI_PORT = "jmxRMIPort";

    public static final String CONF_JMX_HTTP_PORT = "jmxHTTPPort";

    public static final String CONF_JORAM_ID = "joramId";

    public static final String CONF_JORAM_LOGIN = "joramLogin";

    public static final String CONF_JORAM_DOMAIN_PORT = "joramDomainPort";

    public static final String CONF_JORAM_TCP_PORT = "joramTCPPort";

    public static final String CONF_DREAM_TCP_PORT = "dreamTCPPort";

    public static final String CONF_STATE = "state";

    public static final String STATE_STARTED = "STARTED";

    public static final String STATE_STOPPED = "STOPPED";

    /**
     * Retrieve the collection of all the servers configuration.
     *
     * @return a set of container configuration in key/value form
     * @throws PetalsException
     */
    Set<Map<String, String>> retrieveContainersConfiguration()
        throws PetalsException;

    /**
     * Retrieve the collection of all the Started servers configuration.
     *
     * @return a set of container configuration in key/value form
     * @throws PetalsException
     */
    Set<Map<String, String>> retrieveStartedContainersConfiguration() throws PetalsException;

    /**
     * Remove the specified server from the network configuration. <br>
     * The server is unbound, but not shutdown. All ServiceEndpoints activated
     * on this server are unbound. <br>
     * <b>Be very careful when using this operation, the network configuration
     * may become unstable.</b><br>
     * TODO JORAM configuration not updated
     *
     * @param serverName
     * @throws PetalsException
     *             The server is not found in the network configuration, some
     *             Endpoints have not been removed, or problem while accessing
     *             the network configuration
     */
    void removeServerFromNetwork(String serverName) throws PetalsException;

    /**
     * Unload the container from the Petals global environment. All
     * ServiceAssemblies, Components and Share Libraries are stopped, shutdown,
     * and uninstalled. All registered Endpoints are deactivated. Then the
     * container is removed from the domain.
     *
     * @throws Exception
     */
    void shutdownContainer() throws Exception;

    /**
     * Stop the container. The container is still visible in the Petals global
     * environment, but marked as 'stopped'. The registered Endpoints are still
     * visible,too.
     *
     * @throws Exception
     */
    void stopContainer() throws Exception;
}
