/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.kernel.configuration;

import java.io.Serializable;

/**
 * This class holds the global topology information.<br>
 * The configuration service retrieves the configuration when it is started and
 * creates an instance of this class. The instance is exposed on the distributed
 * JNDI server.
 *
 * @author Roland NAUDIN - EbmWebsourcing
 * @since Petals 2.0
 *
 */
public class DomainConfiguration implements Serializable {

    /**
     *
     */
    private static final long serialVersionUID = -4951508542017620408L;

    /**
     * Domain mode enumeration.
     */
    public static enum DomainMode {
        /**
         * Stand-alone mode
         */
        STANDALONE,
        /**
         * Static distributed mode
         */
        STATIC,
        /**
         * Dynamic distributed mode
         */
        DYNAMIC
    };

    // ************ Network *************//

    /**
     * Network domain name.
     */
    private String domain;

    /**
     * The domain mode.
     */
    private DomainMode mode;

    /**
     * Network multicast IP. IP used to transmit PEtALS multicast messages
     */
    private String networkMuticastIp;

    /**
     * Network multicast port. Port used to transmit PEtALS multicast messages
     */
    private int networkMuticastPort;

    // ************* JNDI **************//

    /**
     * JNDI Factory
     */
    private String jndiFactory;

    /**
     * JNDI host
     */
    private String jndiHost;

    /**
     * JNDI Container name
     */
    private String jndiContainer;

    /**
     * JNDI port
     */
    private int jndiPort;

    public String getDomain() {
        return domain;
    }

    public DomainMode getMode() {
        return mode;
    }

    public String getNetworkMuticastIp() {
        return networkMuticastIp;
    }

    public int getNetworkMuticastPort() {
        return networkMuticastPort;
    }

    public String getJndiFactory() {
        return jndiFactory;
    }

    public String getJndiHost() {
        return jndiHost;
    }

    public String getJndiContainer() {
        return jndiContainer;
    }

    public int getJndiPort() {
        return jndiPort;
    }

    protected void setDomain(String domain) {
        this.domain = domain;
    }

    protected void setMode(DomainMode mode) {
        this.mode = mode;
    }

    protected void setNetworkMuticastIp(String networkMuticastIp) {
        this.networkMuticastIp = networkMuticastIp;
    }

    protected void setNetworkMuticastPort(int networkMuticastPort) {
        this.networkMuticastPort = networkMuticastPort;
    }

    protected void setJndiFactory(String jndiFactory) {
        this.jndiFactory = jndiFactory;
    }

    protected void setJndiContainer(String jndiContainer) {
        this.jndiContainer = jndiContainer;
    }

    protected void setJndiHost(String jndiHost) {
        this.jndiHost = jndiHost;
    }

    protected void setJndiPort(int jndiPort) {
        this.jndiPort = jndiPort;
    }

    @Override
    public String toString() {
        StringBuilder toStringBuilder = new StringBuilder();
        toStringBuilder.append("Domain " + this.domain + " configuration :\n");
        toStringBuilder.append("Mode:" + this.mode);
        if (this.networkMuticastIp != null) {
            toStringBuilder.append(" - multicastIP:" + this.networkMuticastIp);
        }
        if (this.networkMuticastPort != 0) {
            toStringBuilder.append(" - multicastPort:"
                + this.networkMuticastPort);
        }
        toStringBuilder.append(" - JNDIFactory:" + this.jndiFactory);
        if (this.jndiHost != null) {
            toStringBuilder.append(" - JNDIHost:" + this.jndiHost);
        }
        if (this.jndiContainer != null) {
            toStringBuilder
                .append(" - JNDIContainerHost:" + this.jndiContainer);
        }
        if (this.jndiPort != 0) {
            toStringBuilder.append(" - JNDIPort:" + this.jndiPort);
        }

        return toStringBuilder.toString();
    }

}
